clear all;
clc;

randn('state',1); rand('state',1); %#ok<RAND>
%% Generate a low-rank matrix
    load('LennaR29.mat')
    M=LennaR29;
    R = rank(M); % Check the true rank of M
    SR = 0.5; % The ratio of observed entries (sample ratio)
    [m n]=size(M);
    idx = randperm(m*n,round(SR*m*n));
    [i,j] = ind2sub([m n],idx);
    obs = sparse(i,j,1,m,n);
    Omega=full(obs);
    Omega=logical(Omega);
    nXX = norm(M(:),'fro');
    nXX_omega=norm(M(Omega),'fro');
    nXX_NOmega=norm(M(~Omega),'fro');
    X = zeros(size(M));
    X(Omega) = double(M(Omega));
    data = M(Omega);
    OmegaSVT=find(Omega==1);
    
    %% parameters to setup the problem
    
    maxIter = 500;
    epsilon = 1e-14;
    mu    = 30;  % The parameter of the L1-norm regularzation term; always set it as 30, you may tune it for some data;
    InitialR = round(1/8*min(size(X))); 
    fr= 1e-3*(SR);  % threhold for rank estimation, it could be even smaller; 
    nrRun = 0;
    tic
    [Z,EstimatedRank,w]  = L1MC_RF(...
        X,...                     % The incomplete matrix whose elements in Omega are used for estimating missing value;
        Omega,...                 % The index set of the obeserved elements: known 1, unkown, 0;
        mu,...                    % The parameter of the L1-norm regularzation term; always set it as 30;
        maxIter,...               % The maximum iterations;
        epsilon,...               % The stop tolerance of the error of approximation Z and input X on observed entries;
        InitialR);                    % The initial maximum rank for target matrix);
elapsed_time = toc;

S=svd(M);     % Singular values of original complete matrix
WeightError = abs(norm(w) - norm(S)) / norm(S); % Error of weights (singular values)
RSE= norm(Z - M) / norm(M);
fprintf('****L1MC-RF****: CPU time: %d\n\n',elapsed_time);
fprintf('****L1MC-RF****: Estimated rank is %d\n\n',EstimatedRank);
fprintf('****L1MC-RF****: Recovery accuracy: RSE= %d\n\n',RSE);
fprintf('****L1MC-RF****: Relative error of weights is: RSE= %d\n\n',WeightError);
